/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include "aiclient.h"

#include "log.h"
#include "support.h"
#include "capstr.h"
#include "game.h"
#include "net.h"
#include "shared.h"
#include "client.h"

char server_host[512];
char name[512];
int server_port;

enum client_states client_state=CLIENT_BOOT_STATE;

void csai_init(int argc, char *argv[])
{
	char err_buf[1024];
	int error;

	error=connect_to_server(user_username(), "localhost", DEFAULT_SOCK_PORT,
							err_buf, sizeof(err_buf));
	if(error)
	{
		printf("error=%s\n",err_buf);
		exit(1);
	}
	else
		freelog(LOG_VERBOSE, "connected");
	set_client_state(CLIENT_PRE_GAME_STATE);
}

void csai_mainloop()
{
	sniff_packets();
}

int main(int argc, char *argv[])
{
	int loglevel;
	char *logfile=NULL;

	init_nls();
	dont_run_as_root(argv[0], "freeciv_client");

	/* set default argument values */
	loglevel=LOG_VERBOSE;
	server_port=DEFAULT_SOCK_PORT;
	mystrlcpy(server_host, "localhost", 512);
	name[0] = '\0';

	/* argv, argc handling */

	log_init(logfile, loglevel, NULL);

	/* after log_init: */
	if (name[0] == '\0') {
		mystrlcpy(name, user_username(), 512);
	}

    freelog(LOG_VERBOSE, "aiclient starting...");

	init_our_capability();

#if 0
	game_init();
#endif
	register_callbacks();
	csai_init(argc, argv);
	csai_mainloop();

	return 0;
}


/**************************************************************************
...
**************************************************************************/


#if 0
/**************************************************************************
...
**************************************************************************/

void user_ended_turn(void)
{
	struct packet_generic_message gen_packet;
	gen_packet.message[0]='\0';

	send_packet_generic_message(&aconnection, PACKET_TURN_DONE, &gen_packet);
}

/**************************************************************************
...
**************************************************************************/
void send_unit_info(struct unit *punit)
{
	struct packet_unit_info info;

	info.id=punit->id;
	info.owner=punit->owner;
	info.x=punit->x;
	info.y=punit->y;
	info.homecity=punit->homecity;
	info.veteran=punit->veteran;
	info.type=punit->type;
	info.movesleft=punit->moves_left;
	info.activity=punit->activity;
	info.activity_target=punit->activity_target;
	info.select_it=0;

	send_packet_unit_info(&aconnection, &info);
}

/**************************************************************************
...
**************************************************************************/
void send_move_unit(struct unit *punit)
{
	struct packet_move_unit move;

	move.unid=punit->id;
	move.x=punit->x;
	move.y=punit->y;

	send_packet_move_unit(&aconnection, &move);
}

/**************************************************************************
...
**************************************************************************/
void send_report_request(enum report_type type)
{
	struct packet_generic_integer pa;
  
	pa.value=type;
	send_packet_generic_integer(&aconnection, PACKET_REPORT_REQUEST, &pa);
}
#endif
/**************************************************************************
...
**************************************************************************/
void set_client_state(enum client_states newstate)
{
	if(client_state!=newstate) {

		/* If changing from pre-game state to _either_ select race
		   or running state, then we have finished getting ruleset data,
		   and should translate data, for joining running game or for
		   selecting nations.  (Want translated nation names in nation
		   select dialog.)
		*/
		if (client_state==CLIENT_PRE_GAME_STATE
			&& (newstate==CLIENT_SELECT_RACE_STATE
				|| newstate==CLIENT_GAME_RUNNING_STATE)) {
#if 0
			translate_data_names();
#endif
		}

		freelog(LOG_VERBOSE, "going from client state %d into %d", 
				client_state,newstate);

		client_state=newstate;

		if(client_state==CLIENT_GAME_RUNNING_STATE) {
#if 0 
			update_research(game.player_ptr);
			role_unit_precalcs();
			boot_help_texts();	/* reboot */
			update_unit_focus();
#endif
		}
		else if(client_state==CLIENT_PRE_GAME_STATE) {
#if 0 
			popdown_all_city_dialogs();
			game_remove_all_players();
			set_unit_focus_no_center(0);
			clear_notify_window();
			idex_init();		/* clear old data */
#endif
		}
	}
	if(client_state==CLIENT_PRE_GAME_STATE)
		/*    gui_server_connect();*/;
}

enum client_states get_client_state(void)
{
	return client_state;
}
