/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_SYS_SIGNAL_H
#include <sys/signal.h>
#endif
#ifdef HAVE_SYS_UIO_H
#include <sys/uio.h>
#endif
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

#include "capstr.h"
#include "fcintl.h"
#include "game.h"
#include "log.h"
#include "mem.h"
#include "packets.h"
#include "support.h"
#include "version.h"
#include "packhand.h"
#include "net.h"
#include "aiclient.h"

struct connection aconnection;
int socket_fd;

void add_net_input(int fd);
void remove_net_input(void);

void handle_packet_input(char *packet, int type)
{
	freelog(LOG_DEBUG, "got packet with type %d",type);

	switch(type) {
		case PACKET_JOIN_GAME_REPLY:
			handle_join_game_reply((struct packet_join_game_reply *)packet);
			break;

		case PACKET_SERVER_SHUTDOWN:
			freelog(LOG_VERBOSE, "server shutdown");
			break;

		case PACKET_BEFORE_NEW_YEAR:
			handle_before_new_year();
			break;

		case PACKET_NEW_YEAR:
			handle_new_year((struct packet_new_year *)packet);
			break;

		case PACKET_UNIT_INFO:
			handle_unit_info((struct packet_unit_info *)packet);
			break;

		case PACKET_MOVE_UNIT:
			handle_move_unit((struct packet_move_unit *)packet);
			break;
    
		case PACKET_TILE_INFO:
			handle_tile_info((struct packet_tile_info *)packet);
			break;

		case PACKET_SELECT_NATION:
			handle_select_nation((struct packet_generic_values *)packet);
			break;

		case PACKET_PLAYER_INFO:
			handle_player_info((struct packet_player_info *)packet);
			break;
    
		case PACKET_GAME_INFO:
			handle_game_info((struct packet_game_info *)packet);
			break;

		case PACKET_MAP_INFO:
			handle_map_info((struct packet_map_info *)packet);
			break;
    
		case PACKET_CHAT_MSG:
			handle_chat_msg((struct packet_generic_message *)packet);
			break;

		case PACKET_PAGE_MSG:
			handle_page_msg((struct packet_generic_message *)packet);
			break;
    
		case PACKET_CITY_INFO:
			handle_city_info((struct packet_city_info *)packet);
			break;

		case PACKET_REMOVE_UNIT:
			handle_remove_unit((struct packet_generic_integer *)packet);
			break;

		case PACKET_REMOVE_CITY:
			handle_remove_city((struct packet_generic_integer *)packet);
			break;
    
		case PACKET_UNIT_COMBAT:
			handle_unit_combat((struct packet_unit_combat *)packet);
			break;

		case PACKET_GAME_STATE:
			handle_game_state(((struct packet_generic_integer *)packet));
			break;

		case PACKET_NUKE_TILE:
			handle_nuke_tile(((struct packet_nuke_tile *)packet));
			break;

		case PACKET_DIPLOMACY_INIT_MEETING:
			handle_diplomacy_init_meeting((struct packet_diplomacy_info *)packet);  
			break;

		case PACKET_DIPLOMACY_CANCEL_MEETING:
			handle_diplomacy_cancel_meeting((struct packet_diplomacy_info *)packet);  
			break;

		case PACKET_DIPLOMACY_CREATE_CLAUSE:
			handle_diplomacy_create_clause((struct packet_diplomacy_info *)packet);  
			break;

		case PACKET_DIPLOMACY_REMOVE_CLAUSE:
			handle_diplomacy_remove_clause((struct packet_diplomacy_info *)packet);  
			break;

		case PACKET_DIPLOMACY_ACCEPT_TREATY:
			handle_diplomacy_accept_treaty((struct packet_diplomacy_info *)packet);  
			break;

		case PACKET_REMOVE_PLAYER:
			handle_remove_player((struct packet_generic_integer *)packet);
			break;

		case PACKET_RULESET_CONTROL:
			handle_ruleset_control((struct packet_ruleset_control *)packet);
			break;

		case PACKET_RULESET_UNIT:
			handle_ruleset_unit((struct packet_ruleset_unit *)packet);
			break;
    
		case PACKET_RULESET_TECH:
			handle_ruleset_tech((struct packet_ruleset_tech *)packet);
			break;
    
		case PACKET_RULESET_BUILDING:
			handle_ruleset_building((struct packet_ruleset_building *)packet);
			break;

		case PACKET_RULESET_TERRAIN:
			handle_ruleset_terrain((struct packet_ruleset_terrain *)packet);
			break;

		case PACKET_RULESET_TERRAIN_CONTROL:
			handle_ruleset_terrain_control((struct terrain_misc *)packet);
			break;

		case PACKET_RULESET_GOVERNMENT:
			handle_ruleset_government((struct packet_ruleset_government *)packet);
			break;
  
		case PACKET_RULESET_GOVERNMENT_RULER_TITLE:
			handle_ruleset_government_ruler_title((struct packet_ruleset_government_ruler_title *)packet);
			break;
  
		case PACKET_RULESET_NATION:
			handle_ruleset_nation((struct packet_ruleset_nation *)packet);
			break;

		case PACKET_RULESET_CITY:
			handle_ruleset_city((struct packet_ruleset_city *)packet);
			break;
    
		case PACKET_INCITE_COST:
			handle_incite_cost((struct packet_generic_values *)packet);
			break;

		case PACKET_CITY_OPTIONS:
			handle_city_options((struct packet_generic_values *)packet);
			break;
    
		case PACKET_SPACESHIP_INFO:
			handle_spaceship_info((struct packet_spaceship_info *)packet);
			break;
    
		case PACKET_CITY_NAME_SUGGESTION:
			handle_city_name_suggestion((struct packet_city_name_suggestion *)packet);
			break;

		case PACKET_SABOTAGE_LIST:
			handle_sabotage_list((struct packet_sabotage_list *)packet);
			break;

		case PACKET_DIPLOMAT_ACTION:
			handle_diplomat_action((struct packet_diplomat_action *)packet);
			break;

		case PACKET_ADVANCE_FOCUS:
			handle_advance_focus((struct packet_generic_integer *)packet);
			break;
    
		default:
			freelog(LOG_FATAL, _("Received unknown packet from server!"));
			exit(1);
			break;
	}

	free(packet);
}

/**************************************************************************
...
**************************************************************************/
static void close_socket_callback(struct connection *pc)
{
	freelog(LOG_NORMAL, "lost connection to server");
	close(pc->sock);
	remove_net_input();
	set_client_state(CLIENT_PRE_GAME_STATE);
}

/**************************************************************************
...
**************************************************************************/
int connect_to_server(char *name, char *hostname, int port,
					  char *errbuf, int n_errbuf)
{
	/* use name to find TCP/IP address of server */
	struct sockaddr_in src;
	struct hostent *ph;
	long address;
	struct packet_req_join_game req;

	if(port==0)
		port=DEFAULT_SOCK_PORT;
  
	if(!hostname)
		hostname="localhost";
  
	if(isdigit((size_t)*hostname)) {
		if((address = inet_addr(hostname)) == -1) {
			mystrlcpy(errbuf, _("Invalid hostname"), n_errbuf);
			return -1;
		}
		src.sin_addr.s_addr = address;
		src.sin_family = AF_INET;
	}
	else if ((ph = gethostbyname(hostname)) == NULL) {
		mystrlcpy(errbuf, _("Failed looking up host"), n_errbuf);
		return -1;
	}
	else {
		src.sin_family = ph->h_addrtype;
		memcpy((char *) &src.sin_addr, ph->h_addr, ph->h_length);
	}
  
	src.sin_port = htons(port);

#ifdef HAVE_SIGPIPE
	/* ignore broken pipes */
	signal (SIGPIPE, SIG_IGN);
#endif
  
	if((aconnection.sock = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
		mystrlcpy(errbuf, mystrerror(errno), n_errbuf);
		return -1;
	}
  
	if(connect(aconnection.sock, (struct sockaddr *) &src, sizeof (src)) < 0) {
		mystrlcpy(errbuf, mystrerror(errno), n_errbuf);
		close(aconnection.sock);
		return -1;
	}

	aconnection.buffer.ndata=0;

  /* gui-dependent details now in gui_main.c: */
	add_net_input(aconnection.sock);
  
	close_socket_set_callback(close_socket_callback);

  /* now send join_request package */

	mystrlcpy(req.short_name, name, MAX_LEN_USERNAME);
	req.major_version=MAJOR_VERSION;
	req.minor_version=MINOR_VERSION;
	req.patch_version=PATCH_VERSION;
	sz_strlcpy(req.version_label, VERSION_LABEL);
	sz_strlcpy(req.capability, our_capability);
	sz_strlcpy(req.name, name);

	send_packet_req_join_game(&aconnection, &req);
  
	return 0;
}

/**************************************************************************
...
**************************************************************************/
void disconnect_from_server(void)
{
	close(aconnection.sock);
	remove_net_input();
	set_client_state(CLIENT_PRE_GAME_STATE);
}  

/**************************************************************************
 This function is called when the client received a
 new input from the server
**************************************************************************/
void input_from_server()
{
	int fid=socket_fd;

	if(read_socket_data(fid, &aconnection.buffer)>=0) {
		int type;
		char *packet;

		while((packet=get_packet_from_connection(&aconnection, &type))) {
			handle_packet_input(packet, type);
		}
	}
	else 
	{
		close_socket_callback(&aconnection);
	}
}

/* common/packets needs this */
/* int unit_flag(Unit_Type_id id, int flag) */
/* { */
/*   assert(flag>=0 && flag<F_LAST); */
/*   return BOOL_VAL(unit_types[id].flags & (1<<flag)); */
/* } */

int unit_flag(int id, int flag)
{
	return 0;
}

void add_net_input(int fd)
{
	socket_fd=fd;
}

void remove_net_input(void)
{
	socket_fd=-1;
}

void sniff_packets()
{
	while(socket_fd!=-1)
		input_from_server();
}
