/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "capability.h"
#include "capstr.h"
#include "events.h"
#include "fcintl.h"
#include "priv_game.h"
#include "priv_government.h"
#include "idex.h"
#include "log.h"
#include "priv_map.h"
#include "mem.h"
#include "priv_nation.h"
#include "packets.h"
//#include "spaceship.h"
#include "support.h"
#include "priv_unit.h"
#include "priv_tech.h"
#include "priv_city.h"
#include "priv_player.h"

#if 0
#include "climisc.h"
#include "clinet.h"		/* aconnection */
#include "control.h"
#include "options.h"
#endif

#include "aiclient.h"
#include "net.h"
#include "packhand.h"
#include "callback.h"

extern int seconds_to_turndone;
extern int last_turn_gold_amount;
extern int did_advance_tech_this_turn;
extern char name[512];

#define LOG_HANDLED LOG_DEBUG
#define LOG_NOT_HANDLED LOG_VERBOSE

/**************************************************************************
...
**************************************************************************/
void handle_join_game_reply(struct packet_join_game_reply *packet)
{
	char msg[MAX_LEN_MSG];
	char *s_capability = aconnection.capability;

	freelog(LOG_HANDLED,"HANDLE_join_game_reply");

	mystrlcpy(s_capability, packet->capability, 
			  sizeof(aconnection.capability));

	if (packet->you_can_join) {
		freelog(LOG_VERBOSE, "join game accept:%s", packet->message);
	} else {
		my_snprintf(msg, sizeof(msg),
					_("You were rejected from the game: %s"), packet->message);
		freelog(LOG_NORMAL,"%s",msg);
	}
	if (strcmp(s_capability, our_capability)==0)
		return;
	my_snprintf(msg, sizeof(msg),
				_("Client capability string: %s"), our_capability);
	freelog(LOG_NORMAL,"%s",msg);
	my_snprintf(msg, sizeof(msg),
				_("Server capability string: %s"), s_capability);
	freelog(LOG_NORMAL,"%s",msg);
}

/**************************************************************************
...
**************************************************************************/
void handle_remove_city(struct packet_generic_integer *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_remove_city");
#if 0
	struct city *pcity=find_city_by_id(packet->value);
	if(pcity!=NULL) client_remove_city(pcity);
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_remove_unit(struct packet_generic_integer *packet)
{
	freelog(LOG_HANDLED,"HANDLE_remove_unit");
	remove_unit(packet->value);
}

/**************************************************************************
...
**************************************************************************/
void handle_nuke_tile(struct packet_nuke_tile *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_nuke_tile");
#if 0
	put_nuke_mushroom_pixmaps(packet->x, packet->y);
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_unit_combat(struct packet_unit_combat *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_unit_combat");
#if 0
	int show_combat = FALSE;
	struct unit *punit0 = find_unit_by_id(packet->attacker_unit_id);
	struct unit *punit1 = find_unit_by_id(packet->defender_unit_id);

	if (punit0 && punit1) {
		if (tile_visible_mapcanvas(punit0->x, punit0->y) &&
			tile_visible_mapcanvas(punit1->x, punit1->y)) {
			show_combat = TRUE;
		} else if (auto_center_on_combat) {
			if (punit0->owner == game.player_idx)
				center_tile_mapcanvas(punit0->x, punit0->y);
			else
				center_tile_mapcanvas(punit1->x, punit1->y);
			show_combat = TRUE;
		}

		if (show_combat) {
			decrease_unit_hp_smooth(punit0, packet->attacker_hp,
									punit1, packet->defender_hp);
		}
	}
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_game_state(struct packet_generic_integer *packet)
{
	freelog(LOG_HANDLED,"HANDLE_game_state");
  
	set_client_state(packet->value);

	if(get_client_state()==CLIENT_GAME_RUNNING_STATE) {
		CALL(cb_game_started_notify,);
		call_choose_science();
#if 0		
		refresh_overview_canvas();
		refresh_overview_viewrect();
		enable_turn_done_button();
		player_set_unit_focus_status(game.player_ptr);

		update_info_label();	/* get initial population right */
		update_unit_focus();
		update_unit_info_label(get_unit_in_focus());

		if(get_unit_in_focus())
			center_tile_mapcanvas(get_unit_in_focus()->x, get_unit_in_focus()->y);
    
		free_intro_radar_sprites();
#endif
	}
}

/**************************************************************************
...
**************************************************************************/
void handle_city_info(struct packet_city_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_city_info");
	update_city(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_new_year(struct packet_new_year *ppacket)
{
	freelog(LOG_HANDLED,"HANDLE_new_year");

	game.year=ppacket->year;
	CALL(cb_new_year_notify,);
}

/**************************************************************************
...
**************************************************************************/
void handle_before_new_year(void)
{
	freelog(LOG_HANDLED,"HANDLE_before_new_year");
}

/**************************************************************************
...
**************************************************************************/
void handle_chat_msg(struct packet_generic_message *packet)
{
	freelog(LOG_HANDLED,"HANDLE_chat_msg");
	if(packet->event==E_WONDER_STARTED ||
	   packet->event==E_WONDER_STOPPED)
		return;

	freelog(LOG_VERBOSE,"msg='%s'",packet->message);
}
 
/**************************************************************************
...
**************************************************************************/
void handle_page_msg(struct packet_generic_message *packet)
{
	freelog(LOG_HANDLED,"HANDLE_page_msg");
	freelog(LOG_VERBOSE,"msg='%s'",packet->message);
}

/**************************************************************************
...
**************************************************************************/
void handle_move_unit(struct packet_move_unit *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_move_unit");
#if 0
	/* should there be something here? - mjd */
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_unit_info(struct packet_unit_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_unit_info");
	update_unit(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_map_info(struct packet_map_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_map_info");

	init_map(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_game_info(struct packet_game_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_game_info");
	update_game_info(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_player_info(struct packet_player_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_player_info");
	update_player_info(p);
}

/**************************************************************************
Ideally the client should let the player choose which type of
modules and components to build, and (possibly) where to extend
structurals.  The protocol now makes this possible, but the
client is not yet that good (would require GUI improvements)
so currently the client choices stuff automatically if there
is anything unplaced.

This function makes a choice (sends spaceship_action) and
returns 1 if we placed something, else 0.

Do things one at a time; the server will send us an updated
spaceship_info packet, and we'll be back here to do anything
which is left.
**************************************************************************/
#if 0
static int spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship)
{
	freelog(LOG_NORMAL,"spaceship_autoplace");

	struct government *g = get_gov_pplayer(pplayer);
	struct packet_spaceship_action packet;
	int i;
  
	if (ship->modules > (ship->habitation + ship->life_support
						 + ship->solar_panels)) {
    
		int nice = government_has_hint(g, G_IS_NICE);
		/* "nice" governments prefer to keep success 100%;
		 * others build habitation first (for score?)  (Thanks Massimo.)
		 */

		packet.action =
			(ship->habitation==0)   ? SSHIP_ACT_PLACE_HABITATION :
			(ship->life_support==0) ? SSHIP_ACT_PLACE_LIFE_SUPPORT :
			(ship->solar_panels==0) ? SSHIP_ACT_PLACE_SOLAR_PANELS :
			((ship->habitation < ship->life_support)
			 && (ship->solar_panels*2 >= ship->habitation + ship->life_support + 1))
			? SSHIP_ACT_PLACE_HABITATION :
			(ship->solar_panels*2 < ship->habitation + ship->life_support)
			? SSHIP_ACT_PLACE_SOLAR_PANELS :
			(ship->life_support<ship->habitation)
			? SSHIP_ACT_PLACE_LIFE_SUPPORT :
			(nice && (ship->life_support <= ship->habitation)
			 && (ship->solar_panels*2 >= ship->habitation + ship->life_support + 1))
			? SSHIP_ACT_PLACE_LIFE_SUPPORT :
			(nice)                  ? SSHIP_ACT_PLACE_SOLAR_PANELS :
			SSHIP_ACT_PLACE_HABITATION;

		if (packet.action == SSHIP_ACT_PLACE_HABITATION) {
			packet.num = ship->habitation + 1;
		} else if(packet.action == SSHIP_ACT_PLACE_LIFE_SUPPORT) {
			packet.num = ship->life_support + 1;
		} else {
			packet.num = ship->solar_panels + 1;
		}
		assert(packet.num<=NUM_SS_MODULES/3);

		send_packet_spaceship_action(&aconnection, &packet);
		return 1;
	}
	if (ship->components > ship->fuel + ship->propulsion) {
		if (ship->fuel <= ship->propulsion) {
			packet.action = SSHIP_ACT_PLACE_FUEL;
			packet.num = ship->fuel + 1;
		} else {
			packet.action = SSHIP_ACT_PLACE_PROPULSION;
			packet.num = ship->propulsion + 1;
		}
		send_packet_spaceship_action(&aconnection, &packet);
		return 1;
	}
	if (ship->structurals > num_spaceship_structurals_placed(ship)) {
		/* Want to choose which structurals are most important.
		   Else we first want to connect one of each type of module,
		   then all placed components, pairwise, then any remaining
		   modules, or else finally in numerical order.
		*/
		int req = -1;
    
		if (!ship->structure[0]) {
			/* if we don't have the first structural, place that! */
			packet.action = SSHIP_ACT_PLACE_STRUCTURAL;
			packet.num = 0;
			send_packet_spaceship_action(&aconnection, &packet);
			return 1;
		}
    
		if (ship->habitation >= 1
			&& !ship->structure[modules_info[0].required]) {
			req = modules_info[0].required;
		} else if (ship->life_support >= 1
				   && !ship->structure[modules_info[1].required]) {
			req = modules_info[1].required;
		} else if (ship->solar_panels >= 1
				   && !ship->structure[modules_info[2].required]) {
			req = modules_info[2].required;
		} else {
			int i;
			for(i=0; i<NUM_SS_COMPONENTS; i++) {
				if ((i%2==0 && ship->fuel > (i/2))
					|| (i%2==1 && ship->propulsion > (i/2))) {
					if (!ship->structure[components_info[i].required]) {
						req = components_info[i].required;
						break;
					}
				}
			}
		}
		if (req == -1) {
			for(i=0; i<NUM_SS_MODULES; i++) {
				if ((i%3==0 && ship->habitation > (i/3))
					|| (i%3==1 && ship->life_support > (i/3))
					|| (i%3==2 && ship->solar_panels > (i/3))) {
					if (!ship->structure[modules_info[i].required]) {
						req = modules_info[i].required;
						break;
					}
				}
			}
		}
		if (req == -1) {
			for(i=0; i<NUM_SS_STRUCTURALS; i++) {
				if (!ship->structure[i]) {
					req = i;
					break;
				}
			}
		}
		/* sanity: */
		assert(req!=-1);
		assert(!ship->structure[req]);
    
		/* Now we want to find a structural we can build which leads to req.
		   This loop should bottom out, because everything leads back to s0,
		   and we made sure above that we do s0 first.
		*/
		while(!ship->structure[structurals_info[req].required]) {
			req = structurals_info[req].required;
		}
		packet.action = SSHIP_ACT_PLACE_STRUCTURAL;
		packet.num = req;
		send_packet_spaceship_action(&aconnection, &packet);
		return 1;
	}
	return 0;
}
#endif
/**************************************************************************
...
**************************************************************************/
void handle_spaceship_info(struct packet_spaceship_info *p)
{
	freelog(LOG_NOT_HANDLED,"handle_spaceship_info");
#if 0
	int i;
	struct player *pplayer=&game.players[p->player_num];
	struct player_spaceship *ship = &pplayer->spaceship;
  
	ship->state        = p->sship_state;
	ship->structurals  = p->structurals;
	ship->components   = p->components;
	ship->modules      = p->modules;
	ship->fuel         = p->fuel;
	ship->fuel         = p->fuel;
	ship->propulsion   = p->propulsion;
	ship->habitation   = p->habitation;
	ship->life_support = p->life_support;
	ship->solar_panels = p->solar_panels;
	ship->launch_year  = p->launch_year;
	ship->population   = p->population;
	ship->mass         = p->mass;
	ship->support_rate = p->support_rate;
	ship->energy_rate  = p->energy_rate;
	ship->success_rate = p->success_rate;
	ship->travel_time  = p->travel_time;
  
	for(i=0; i<NUM_SS_STRUCTURALS; i++) {
		ship->structure[i] = p->structure[i]-'0';
	}

	if (pplayer != game.player_ptr) {
		refresh_spaceship_dialog(pplayer);
		return;
	}
	update_menus();

	if (!spaceship_autoplace(pplayer, ship)) {
		refresh_spaceship_dialog(pplayer);
	}
#endif
}

/**************************************************************************
This was once very ugly...
**************************************************************************/
void handle_tile_info(struct packet_tile_info *p)
{
	freelog(LOG_HANDLED,"HANDLE_tile_info");
	update_tile(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_remove_player(struct packet_generic_integer *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_remove_player");
#if 0
	client_remove_player(packet->value);
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_select_nation(struct packet_generic_values *p)
{
	int i, mybits;
	struct packet_alloc_nation packet;

	freelog(LOG_HANDLED,"HANDLE_select_nation value1=%x value2=%x",
			p->value1,p->value2);

	/* status handling */
	if(get_client_state()==CLIENT_SELECT_RACE_STATE) {
		if(p->value2 == 0xffff) {
			set_client_state(CLIENT_WAITING_FOR_GAME_START_STATE);
			return;
		}
	}
	else if(get_client_state()==CLIENT_PRE_GAME_STATE) {
		set_client_state(CLIENT_SELECT_RACE_STATE);
	}
	else
	{
		freelog(LOG_VERBOSE, "got a select nation packet in "
				"an incompatible state");
		return;
	}

	/* update is_selectable for all nations */
	mybits=p->value1;

	for(i=0; i<game.playable_nation_count && i<32; i++) 
	{
		find_nation_by_id(i)->is_selectable= !(mybits&1);
		mybits>>=1;
	}

	mybits=p->value2;

	for(i=32; i<game.playable_nation_count; i++) 
	{
		find_nation_by_id(i)->is_selectable= !(mybits&1);
		mybits>>=1;
	}

	CALL(cb_select_nation, &packet.nation_no, &packet.is_male,
		 &packet.city_style, packet.name);

	send_packet_alloc_nation(&aconnection, &packet);
}

/**************************************************************************
  Take arrival of ruleset control packet to indicate that
  current allocated governments should be free'd, and new
  memory allocated for new size. The same for nations.
**************************************************************************/
void handle_ruleset_control(struct packet_ruleset_control *packet)
{
	int i;

	freelog(LOG_HANDLED,"HANDLE_ruleset_control");
      
	game.aqueduct_size = packet->aqueduct_size;
	game.sewer_size = packet->sewer_size;
	game.add_to_size_limit = packet->add_to_size_limit;
	game.government_when_anarchy = packet->government_when_anarchy;
	game.default_government = packet->default_government;
	game.playable_nation_count = packet->playable_nation_count;
  
	game.rtech.get_bonus_tech = packet->rtech.get_bonus_tech;
	game.rtech.cathedral_plus = packet->rtech.cathedral_plus;
	game.rtech.cathedral_minus = packet->rtech.cathedral_minus;
	game.rtech.colosseum_plus = packet->rtech.colosseum_plus;
	game.rtech.temple_plus = packet->rtech.temple_plus;

	for(i=0; i<MAX_NUM_TECH_LIST; i++) {
		game.rtech.partisan_req[i]  = packet->rtech.partisan_req[i];
		freelog(LOG_DEBUG, "techl %d: %d", i, game.rtech.partisan_req[i]);
	}

	free_governments();
	alloc_governments(packet->government_count);

	free_nations();
	alloc_nations(packet->nation_count);

	free_unit_types();
	alloc_unit_types(packet->num_unit_types);
	
	free_techs();
	alloc_techs(packet->num_tech_types);

	free_city_styles();
	alloc_city_styles(packet->style_count);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_unit(struct packet_ruleset_unit *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_unit");
	add_unit_type(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_tech(struct packet_ruleset_tech *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_tech");

	add_tech(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_building(struct packet_ruleset_building *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_building");
	add_improvement_type(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_government(struct packet_ruleset_government *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_government");
	add_government(p);
}

void handle_ruleset_government_ruler_title (struct packet_ruleset_government_ruler_title *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_government_ruler_title ");
	add_ruler_title(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_terrain");
	add_tile_type(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_terrain_control(struct terrain_misc *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_terrain_control");
	update_terrain_control(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_nation(struct packet_ruleset_nation *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_nation");
	add_nation(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_ruleset_city(struct packet_ruleset_city *p)
{
	freelog(LOG_HANDLED,"HANDLE_ruleset_city");
	add_city_style(p);
}

/**************************************************************************
...
**************************************************************************/
void handle_incite_cost(struct packet_generic_values *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_incite_cost");
#if 0
	struct city *pcity=find_city_by_id(packet->id);
	struct unit *punit=find_unit_by_id(packet->id);

	if(pcity) {
		pcity->incite_revolt_cost = packet->value1;
		if(!game.player_ptr->ai.control || ai_popup_windows)
			popup_incite_dialog(pcity);
		return;
	}

	if(punit) {
		punit->bribe_cost = packet->value1;
		if(!game.player_ptr->ai.control || ai_popup_windows)
			popup_bribe_dialog(punit);
	}
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_city_options(struct packet_generic_values *preq)
{
	freelog(LOG_NOT_HANDLED,"handle_city_options");
#if 0
	struct city *pcity = find_city_by_id(preq->value1);
  
	if (!pcity || pcity->owner != game.player_idx) return;
	pcity->city_options = preq->value2;
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_city_name_suggestion(struct packet_city_name_suggestion *packet)
{
	struct unit *punit;
    struct packet_unit_request req;

	freelog(LOG_HANDLED,"HANDLE_city_name_suggestion");

	punit = find_unit_by_id(packet->id);
	if(!punit)
		return;

	CALL(cb_name_new_city, punit->id, packet->name, req.name);

    req.unit_id=punit->id;
    send_packet_unit_request(&aconnection, &req, PACKET_UNIT_BUILD_CITY);
}

/**************************************************************************
...
**************************************************************************/
void handle_diplomat_action(struct packet_diplomat_action *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomat_action");
#if 0
	struct unit *pdiplomat=player_find_unit_by_id(game.player_ptr, packet->diplomat_id);

	if (!pdiplomat) {
		freelog(LOG_NORMAL, "Received bad diplomat id %d in handle_diplomat_action()",
				packet->diplomat_id);
		return;
	}

	switch(packet->action_type) {
		case DIPLOMAT_CLIENT_POPUP_DIALOG:
			process_diplomat_arrival(pdiplomat, packet->target_id);
			break;
		default:
			freelog(LOG_NORMAL, "Received bad action %d in handle_diplomat_action()",
					packet->action_type);
			break;
	}
#endif
}

/**************************************************************************
...
**************************************************************************/
void handle_sabotage_list(struct packet_sabotage_list *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_sabotage_list");
#if 0
	struct unit *punit = player_find_unit_by_id(game.player_ptr, packet->diplomat_id);
	struct city *pcity = find_city_by_id(packet->city_id);

	if (punit && pcity) {
		int i;
		for(i=0; i<B_LAST; i++)
			pcity->improvements[i] = (packet->improvements[i]=='1') ? 1 : 0;

		popup_sabotage_dialog(pcity);
	}
#endif
}


void handle_advance_focus(struct packet_generic_integer *packet)
{
	freelog(LOG_NOT_HANDLED,"handle_advance_focus");
#if 0
	struct unit *punit = find_unit_by_id(packet->value);
	if (punit && punit_focus == punit)
		advance_unit_focus();
#endif
}

void handle_diplomacy_accept_treaty(struct packet_diplomacy_info *pa)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomacy_accept_treaty");
#if 0
	struct Diplomacy_dialog *pdialog;
  
	if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
									  &game.players[pa->plrno1]))) {
		if(pa->plrno_from==game.player_idx)
			pdialog->treaty.accept0=!pdialog->treaty.accept0;
		else
			pdialog->treaty.accept1=!pdialog->treaty.accept1;
		update_diplomacy_dialog(pdialog);
	}
#endif
}

void handle_diplomacy_init_meeting(struct packet_diplomacy_info *pa)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomacy_init_meeting");
#if 0
	popup_diplomacy_dialog(&game.players[pa->plrno0], 
						   &game.players[pa->plrno1]);
#endif
}

void handle_diplomacy_cancel_meeting(struct packet_diplomacy_info *pa)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomacy_cancel_meeting");
#if 0
	struct Diplomacy_dialog *pdialog;
  
	if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
									  &game.players[pa->plrno1])))
		close_diplomacy_dialog(pdialog);
#endif
}

void handle_diplomacy_create_clause(struct packet_diplomacy_info *pa)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomacy_create_clause");
#if 0
	struct Diplomacy_dialog *pdialog;
  
	if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
									  &game.players[pa->plrno1]))) {
		add_clause(&pdialog->treaty, &game.players[pa->plrno_from],
				   pa->clause_type, pa->value);
		update_diplomacy_dialog(pdialog);
	}
#endif
}

void handle_diplomacy_remove_clause(struct packet_diplomacy_info *pa)
{
	freelog(LOG_NOT_HANDLED,"handle_diplomacy_remove_clause");
#if 0
	struct Diplomacy_dialog *pdialog;

	if((pdialog=find_diplomacy_dialog(&game.players[pa->plrno0],
									  &game.players[pa->plrno1]))) {
		remove_clause(&pdialog->treaty, &game.players[pa->plrno_from],
					  pa->clause_type, pa->value);
    
		update_diplomacy_dialog(pdialog);
	}
#endif
}

