
#include <assert.h>

#include "priv_government.h"
#include "mem.h"
#include "log.h"
#include "support.h"

struct government *governments=NULL;
int governments_count=0;

void free_governments()
{
	int i;

	if(!governments || !governments_count)
		return;

	for(i=0; i<governments_count; i++) 
	{
		if(governments[i].ruler_titles)
			free(governments[i].ruler_titles);
		if(governments[i].helptext)
			free(governments[i].helptext);
	}
	free(governments);
	governments=NULL;
	governments_count=0;
}

void alloc_governments(int size)
{
	int i;

	governments = fc_calloc(size, sizeof(struct government));
	governments_count=size;

	for(i=0; i<size; i++) {
		governments[i].ruler_titles = NULL;
		governments[i].helptext = NULL;
	}
}

void add_government(struct packet_ruleset_government *p)
{
	struct government *gov;

	if (p->id < 0 || p->id >= governments_count) {
		freelog(LOG_NORMAL,
				"Received bad government id %d in handle_ruleset_government",
				p->id);
		return;
	}
	gov = &governments[p->id];

	gov->id				   = p->id;
	gov->required_tech     = p->required_tech;
	gov->max_rate          = p->max_rate;
	gov->civil_war         = p->civil_war;
	gov->martial_law_max   = p->martial_law_max;
	gov->martial_law_per   = p->martial_law_per;
	gov->empire_size_mod   = p->empire_size_mod;
	gov->empire_size_inc   = p->empire_size_inc;
	gov->rapture_size      = p->rapture_size;
  
	gov->unit_happy_cost_factor  = p->unit_happy_cost_factor;
	gov->unit_shield_cost_factor = p->unit_shield_cost_factor;
	gov->unit_food_cost_factor   = p->unit_food_cost_factor;
	gov->unit_gold_cost_factor   = p->unit_gold_cost_factor;
  
	gov->free_happy          = p->free_happy;
	gov->free_shield         = p->free_shield;
	gov->free_food           = p->free_food;
	gov->free_gold           = p->free_gold;

	gov->trade_before_penalty   = p->trade_before_penalty;
	gov->shields_before_penalty = p->shields_before_penalty;
	gov->food_before_penalty    = p->food_before_penalty;

	gov->celeb_trade_before_penalty   = p->celeb_trade_before_penalty;
	gov->celeb_shields_before_penalty = p->celeb_shields_before_penalty;
	gov->celeb_food_before_penalty    = p->celeb_food_before_penalty;

	gov->trade_bonus         = p->trade_bonus;
	gov->shield_bonus        = p->shield_bonus;
	gov->food_bonus          = p->food_bonus;

	gov->celeb_trade_bonus   = p->celeb_trade_bonus;
	gov->celeb_shield_bonus  = p->celeb_shield_bonus;
	gov->celeb_food_bonus    = p->celeb_food_bonus;

	gov->corruption_level    = p->corruption_level;
	gov->corruption_modifier = p->corruption_modifier;
	gov->fixed_corruption_distance = p->fixed_corruption_distance;
	gov->corruption_distance_factor = p->corruption_distance_factor;
	gov->extra_corruption_distance = p->extra_corruption_distance;

	gov->flags               = p->flags;
	gov->hints               = p->hints;
	gov->num_ruler_titles    = p->num_ruler_titles;
    
	sz_strlcpy(gov->name, p->name);
	sz_strlcpy(gov->graphic_str, p->graphic_str);
	sz_strlcpy(gov->graphic_alt, p->graphic_alt);

	gov->ruler_titles = fc_calloc(gov->num_ruler_titles,
								  sizeof(struct ruler_title));
  
	gov->helptext = p->helptext;	/* pointer assignment */
	freelog(LOG_DEBUG,"added '%s' under index %d",p->name,p->id);
}

void add_ruler_title(struct packet_ruleset_government_ruler_title *p)
{
	struct government *gov;

	if(p->gov < 0 || p->gov >= governments_count) {
		freelog(LOG_NORMAL, "Received bad government num %d for title", p->gov);
		return;
	}
	gov = &governments[p->gov];
	if(p->id < 0 || p->id >= gov->num_ruler_titles) {
		freelog(LOG_NORMAL, "Received bad ruler title num %d for %s title",
				p->id, gov->name);
		return;
	}
	gov->ruler_titles[p->id].nation = p->nation;
	sz_strlcpy(gov->ruler_titles[p->id].male_title, p->male_title);
	sz_strlcpy(gov->ruler_titles[p->id].female_title, p->female_title);
	freelog(LOG_DEBUG,"added '%s'/'%s' to '%s'",p->male_title, 
			p->female_title,gov->name);
}

struct government *find_government_by_city(int city_id)
{
	return find_government_by_id(find_player_by_id
								 (find_city_by_id
								  (city_id)->owner)->government);
}

int government_exists(int government_id)
{
	return find_government_by_id(government_id)!=NULL;
}

struct government *find_government_by_id(int government_id)
{
	if(government_id < 0 || government_id >= governments_count)
		return NULL;
	return &governments[government_id];
}

int government_has_flag(int government_id,
						enum government_flag_id flag)
{
	struct government *gov=find_government_by_id(government_id);

	assert(gov);
	assert(flag>=G_FIRST_FLAG && flag<G_LAST_FLAG);

	return gov->flags & (1<<flag);
}

struct government *find_government_by_name(char *name)
{
	int i;

	for(i=0;i<governments_count;i++)
		if(strcmp(governments[i].name,name)==0)
			return &governments[i];
	return NULL;
}
