
#include <assert.h>

#include "priv_map.h"
#include "log.h"
#include "support.h"
#include "mem.h"
#include "astring.h"

struct terrain_misc terrain_control;
struct tile_type tile_types[T_LAST];
int tile_types_count=T_LAST;
struct civ_map map={0,0,0,NULL};
static struct tile void_tile;

#define tile_is_known(x,y) (find_tile_by_position(x,y)->known)
#define map_get_terrain(x,y) (find_tile_by_position(x,y)->terrain)
#define map_get_special(x,y) (find_tile_by_position(x,y)->special)

void update_terrain_control(struct terrain_misc *p)
{
	terrain_control.river_style = p->river_style;
	terrain_control.may_road = p->may_road;
	terrain_control.may_irrigate = p->may_irrigate;
	terrain_control.may_mine = p->may_mine;
	terrain_control.may_transform = p->may_transform;
	terrain_control.ocean_reclaim_requirement = p->ocean_reclaim_requirement;
	terrain_control.river_move_mode = p->river_move_mode;
	terrain_control.river_defense_bonus = p->river_defense_bonus;
	terrain_control.river_trade_incr = p->river_trade_incr;
	free(terrain_control.river_help_text);
	terrain_control.river_help_text = p->river_help_text; /* malloc'ed string */
	terrain_control.fortress_defense_bonus = p->fortress_defense_bonus;
	terrain_control.road_superhighway_trade_bonus = p->road_superhighway_trade_bonus;
	terrain_control.rail_food_bonus = p->rail_food_bonus;
	terrain_control.rail_shield_bonus = p->rail_shield_bonus;
	terrain_control.rail_trade_bonus = p->rail_trade_bonus;
	terrain_control.farmland_supermarket_food_bonus = p->farmland_supermarket_food_bonus;
	terrain_control.pollution_food_penalty = p->pollution_food_penalty;
	terrain_control.pollution_shield_penalty = p->pollution_shield_penalty;
	terrain_control.pollution_trade_penalty = p->pollution_trade_penalty;
}

void add_tile_type(struct packet_ruleset_terrain *p)
{
	struct tile_type *t;

	if (p->id < T_FIRST || p->id >= T_COUNT) {
		freelog(LOG_NORMAL,
				"Received bad terrain id %d in handle_ruleset_terrain",
				p->id);
		return;
	}
	t = &(tile_types[p->id]);

	sz_strlcpy(t->terrain_name, p->terrain_name);
	t->movement_cost = p->movement_cost;
	t->defense_bonus = p->defense_bonus;
	t->food = p->food;
	t->shield = p->shield;
	t->trade = p->trade;
	sz_strlcpy(t->special_1_name, p->special_1_name);
	t->food_special_1 = p->food_special_1;
	t->shield_special_1 = p->shield_special_1;
	t->trade_special_1 = p->trade_special_1;
	sz_strlcpy(t->special_2_name, p->special_2_name);
	t->food_special_2 = p->food_special_2;
	t->shield_special_2 = p->shield_special_2;
	t->trade_special_2 = p->trade_special_2;

	t->road_time = p->road_time;
	t->road_trade_incr = p->road_trade_incr;
	t->irrigation_result = p->irrigation_result;
	t->irrigation_food_incr = p->irrigation_food_incr;
	t->irrigation_time = p->irrigation_time;
	t->mining_result = p->mining_result;
	t->mining_shield_incr = p->mining_shield_incr;
	t->mining_time = p->mining_time;
	t->transform_result = p->transform_result;
	t->transform_time = p->transform_time;
  
	free(t->helptext);
	t->helptext = p->helptext;	/* pointer assignment */
  
	freelog(LOG_DEBUG,"added '%s' under index %d",p->terrain_name,p->id);
}

void tile_init(struct tile *ptile)
{
	ptile->terrain=T_UNKNOWN;
	ptile->special=S_NO_SPECIAL;
	ptile->known=0;
#if 0
	ptile->sent=0;
	ptile->city=NULL;
	unit_list_init(&ptile->units);
	ptile->worked = NULL; /* pointer to city working tile */
	ptile->assigned = 0; /* bitvector */
#endif
}

void map_allocate(void)
{
  int x,y;

  freelog(LOG_DEBUG, "map_allocate (was %p) (%d,%d)",
		  map.tiles, map.xsize, map.ysize);
  
  map.tiles=fc_realloc(map.tiles, map.xsize*map.ysize*sizeof(struct tile));
  for(y=0; y<map.ysize; y++) {
	  for(x=0; x<map.xsize; x++) {
		  tile_init(find_tile_by_position(x, y));
	  }
  }
}

#define MAX_NUM_CONT 32767   /* max portable value in signed short */

/* Static data used to keep track of continent numbers in client:
 * maximum value used, and array of recycled values which can be used.
 * (Could used signed short, but int is easier and storage here will
 * be fairly small.)
 */
static int max_cont_used = 0;
static struct athing recyc_conts;   /* .n is number available */
static int recyc_init = 0;	    /* for first init of recyc_conts */
static int *recyc_ptr = NULL;	    /* set to recyc_conts.ptr (void* vs int*) */

/**************************************************************************
  Initialise above data, or re-initialize (eg, new map).
**************************************************************************/
void climap_init_continents(void)
{
  if (!recyc_init) {
    /* initialize with size first time: */
    ath_init(&recyc_conts, sizeof(int));
    recyc_init = 1;
  }
  ath_free(&recyc_conts);
  recyc_ptr = NULL;
  max_cont_used = 0;
}

/**************************************************************************
  Recycle a continent number.
  (Ie, number is no longer used, and may be re-used later.)
**************************************************************************/
static void recycle_continent_num(int cont)
{
  assert(cont>0 && cont<=max_cont_used);          /* sanity */
  ath_minnum(&recyc_conts, recyc_conts.n+1);
  recyc_ptr = recyc_conts.ptr;
  recyc_ptr[recyc_conts.n-1] = cont;
  freelog(LOG_DEBUG, "clicont: recycling %d (max %d recyc %d)",
	  cont, max_cont_used, recyc_conts.n);
}

/**************************************************************************
  Obtain an unused continent number: a recycled number if available,
  or increase the maximum.
**************************************************************************/
static int new_continent_num(void)
{
  int ret;
  
  if (recyc_conts.n>0) {
    ret = recyc_ptr[--recyc_conts.n];
  } else {
    assert(max_cont_used<MAX_NUM_CONT);
    ret = ++max_cont_used;
  }
  freelog(LOG_DEBUG, "clicont: new %d (max %d, recyc %d)",
	  ret, max_cont_used, recyc_conts.n);
  return ret;
}

/**************************************************************************
  Recursively renumber the client continent at (x,y) with continent
  number 'new'.  Ie, renumber (x,y) tile and recursive adjacent
  known land tiles with the same previous continent ('old').
**************************************************************************/
static void climap_renumber_continent(int x, int y, int newnumber)
{
  int i, j, old;

  if (y<0 || y>=map.ysize) return;
  x = map_adjust_x(x);

  old = find_tile_by_position(x,y)->continent;

  /* some sanity checks: */
  assert(tile_is_known(x,y) >= TILE_KNOWN_FOGGED);
  assert(map_get_terrain(x,y) != T_OCEAN);
  assert(old>0 && old<=max_cont_used);
  
  find_tile_by_position(x,y)->continent=newnumber;
  for(i=x-1; i<=x+1; i++) {
    for(j=y-1; j<=y+1; j++) {
      if(!(i==x && j==y) && j>=0 && j<map.ysize
	 && tile_is_known(i,j)>=TILE_KNOWN_FOGGED
	 && map_get_terrain(i,j)!=T_OCEAN
	 && find_tile_by_position(x,y)->continent==old) {
	climap_renumber_continent(i,j,newnumber);
      }
    }
  }
}

/**************************************************************************
  Update continent numbers when (x,y) becomes known (if (x,y) land).
  Check neighbouring known land tiles: the first continent number
  found becomes the continent value of this tile.  Any other continents
  found are numbered to this continent (ie, continents are merged)
  and previous continent values are recycled.  If no neighbours are
  numbered, use a new number. 
**************************************************************************/
void climap_update_continents(int x, int y)
{
  struct tile *ptile = find_tile_by_position(x,y);
  int i, j, con, this_con;

  if(ptile->terrain == T_OCEAN) return;
  this_con = -1;
  for(i=x-1; i<=x+1; i++) {
    for(j=y-1; j<=y+1; j++) {
      if(!(i==x && j==y) && j>=0 && j<map.ysize
	 && tile_is_known(i,j)>=TILE_KNOWN_FOGGED
	 && map_get_terrain(i,j)!=T_OCEAN) {
	con = find_tile_by_position(i,j)->continent;
	if(con>0) {
	  if(this_con==-1) {
	    ptile->continent = this_con = con;
	  } else if(con != this_con) {
	    freelog(LOG_DEBUG, "renumbering client continent %d to %d at (%d %d)",
		    con, this_con, x, y);
	    climap_renumber_continent(i, j, this_con);
	    recycle_continent_num(con);
	  }
	}
      }
    }
  }
  if(this_con==-1) {
    ptile->continent = new_continent_num();
    freelog(LOG_DEBUG, "new client continent %d at (%d %d)",
	    ptile->continent, x, y);
  }
}

void init_map(struct packet_map_info *p)
{
	map.xsize=p->xsize;
	map.ysize=p->ysize;
	map.is_earth=p->is_earth;

	map_allocate();
	climap_init_continents();
}

#define map_adjust_x(X) \
 (((X)<0) ?  (X)+map.xsize : (((X)>=map.xsize) ? (X)-map.xsize : (X)))

struct tile *find_tile_by_position(int x, int y)
{
	if(y<0 || y>=map.ysize)
	{
		assert(0);
		return &void_tile; /* accurate fix by Syela */
	}
	else
		return map.tiles+map_adjust_x(x)+y*map.xsize;
}

void update_tile(struct packet_tile_info *p)
{
	struct tile *ptile = find_tile_by_position(p->x, p->y);
	enum tile_terrain_type old_terrain = ptile->terrain;
	enum known_type old_known = ptile->known;
	int tile_changed = 0;

	if (ptile->terrain != p->type) { /*terrain*/
		tile_changed = 1;
		ptile->terrain = p->type;
	}
	if (ptile->special != p->special) { /*add-on*/
		tile_changed = 1;
		ptile->special = p->special;
	}
	ptile->known = p->known;

	/* update continents */
	if ((p->known >= TILE_KNOWN_FOGGED &&
		 old_known == TILE_UNKNOWN && ptile->terrain != T_OCEAN) ||
		((old_terrain == T_OCEAN) && (ptile->terrain != T_OCEAN))) {
		/* new knowledge or new land -- update can handle incrementally */
		climap_update_continents(p->x, p->y);
	} else if (old_known >= TILE_KNOWN_FOGGED &&
			   ((old_terrain != T_OCEAN) && (ptile->terrain == T_OCEAN))) {
		/* land changed into ocean -- rebuild continents map from scratch */
		int x, y;
		for (y = 0; y < map.ysize; y++)
			for (x = 0; x < map.xsize; x++)
				find_tile_by_position(x,y)->continent=0;
		climap_init_continents();
		for (y = 0; y < map.ysize; y++)
			for (x = 0; x < map.xsize; x++)
				if ((tile_is_known(x, y) >= TILE_KNOWN_FOGGED) &&
					(map_get_terrain(x, y) != T_OCEAN))
					climap_update_continents(x, y);
	}
}

int same_pos(int x1, int y1, int x2, int y2)
{
  return (map_adjust_x(x1) == map_adjust_x(x2)
	  && map_adjust_y(y1) == map_adjust_y(y2)); 
}

int map_distance(int x0, int y0, int x1, int y1)
{
	int tmp;

	if(y0>y1)
	{
		tmp=y0;
		y0=y1;
		y1=tmp;
	}
	tmp = map_adjust_x(x0-x1);
	return MIN(tmp, map.xsize-tmp)+y1-y0;
}

int is_tiles_adjacent(int x0, int y0, int x1, int y1)
{
	x0 = map_adjust_x(x0);
	x1 = map_adjust_x(x1);
	if (same_pos(x0, y0, x1, y1))
		return 1;
	return (((x0<=x1+1 && x0>=x1-1) || 
			 (x0==0 && x1==map.xsize-1) ||
			 (x0==map.xsize-1 && x1==0)) && (y0<=y1+1 && y0>=y1-1));
}

struct tile_type *find_tile_type_by_id(enum tile_terrain_type type)
{
	if(type<0 || type >=tile_types_count)
		return NULL;

	return &tile_types[type];
}

int is_terrain_near_tile(int x, int y, enum tile_terrain_type t)
{
	if (find_tile_by_position(x-1, y+1)->terrain==t) return 1;
	if (find_tile_by_position(x+1, y-1)->terrain==t) return 1;
	if (find_tile_by_position(x-1, y-1)->terrain==t) return 1;
	if (find_tile_by_position(x+1, y+1)->terrain==t) return 1;
	if (find_tile_by_position(x,   y+1)->terrain==t)   return 1;
	if (find_tile_by_position(x-1,   y)->terrain==t)   return 1;
	if (find_tile_by_position(x+1,   y)->terrain==t)   return 1;
	if (find_tile_by_position(x,   y-1)->terrain==t)   return 1;
	return 0;
}

int is_irrigation_water_adjacent(int x, int y)
{
	struct tile *ptile;
	int x1,y1;

	for (y1=y-1;y1<y+2;y1++) 
		for (x1=x-1;x1<x+2;x1++) 
		{
			ptile=find_tile_by_position(x1,y1);
			
			if (ptile->terrain==T_OCEAN || 
				ptile->terrain==T_RIVER || 
				ptile->special&S_RIVER ||
				ptile->special&S_IRRIGATION)
				return 1;
		}
	return 0;
}

int get_tile_infrastructure_set(enum tile_special_type tile_special)
{
	return (tile_special &
			(S_ROAD | S_RAILROAD | S_IRRIGATION | 
			 S_FARMLAND | S_MINE | S_FORTRESS | S_AIRBASE));
}

int get_unit_tile_pillage_set(int x, int y)
{
	int tgt_ret = S_NO_SPECIAL;
	struct unit *punits[10];
	int units,i;

	units=find_units_by_position(x,y,10,punits);
	for(i=0;i<units;i++)
	{
		if(punits[i]->activity==ACTIVITY_PILLAGE)
			tgt_ret |= punits[i]->activity_target;
	}
	return tgt_ret;
}

int get_preferred_pillage(int pset)
{
	if(pset&S_FARMLAND)
		return S_FARMLAND;
	if(pset&S_IRRIGATION)
		return S_IRRIGATION;
	if(pset&S_MINE)
		return S_MINE;
	if(pset&S_FORTRESS)
		return S_FORTRESS;
	if(pset&S_AIRBASE)
		return S_AIRBASE;
	if(pset&S_RAILROAD)
		return S_RAILROAD;
	if(pset&S_ROAD)
		return S_ROAD;
	return S_NO_SPECIAL;
}

int map_move_cost(int unit_id,int x2, int y2)
{
	struct unit *punit;
	int x1,y1;
	struct tile *t1,*t2;

	punit=find_unit_by_id(unit_id);
	x1=punit->x;
	y1=punit->y;
	t1=find_tile_by_position(x1,y1);
	t2=find_tile_by_position(x2,y2);

	
	if (punit && !punit->is_ground_unit)
		return 3;
	if( (t1->special&S_RAILROAD) && (t2->special&S_RAILROAD) )
		return 0;
	if (punit && unit_has_flag(unit_id, F_IGTER))
		return 1;
	if( (t1->special&S_ROAD) && (t2->special&S_ROAD) )
		return 1;

	if( ( (t1->terrain==T_RIVER) && (t2->terrain==T_RIVER) ) ||
		( (t1->special&S_RIVER) && (t2->special&S_RIVER) ) ) 
	{
		int cardinal_move;

		cardinal_move = (y1==y2 || map_adjust_x(x1)==map_adjust_x(x2));
		
		switch (terrain_control.river_move_mode) {
			case RMV_NORMAL:
				break;
			case RMV_FAST_STRICT:
				if (cardinal_move)
					return 1;
				break;
			case RMV_FAST_RELAXED:
				if (cardinal_move)
					return 1;
				else
					return 2;
			case RMV_FAST_ALWAYS:
				return 1;
			default:
				break;
		}
	}

	return(find_tile_type_by_id(t2->terrain)->movement_cost*3);
}
