
#include <assert.h>

#include "mem.h"
#include "priv_nation.h"
#include "log.h"
#include "support.h"

struct nation_type *nations=NULL;
int nations_count=0;

void free_nations()
{
	if(!nations || !nations_count)
		return;

	free(nations);
	nations=NULL;
	nations_count=0;
}

void alloc_nations(int size)
{
	nations = fc_calloc(size, sizeof(struct nation_type));
	nations_count=size;
}

void add_nation(struct packet_ruleset_nation *p)
{
	int i;
	struct nation_type *pl;

	if(p->id < 0 || p->id >= nations_count || p->id >= MAX_NUM_NATIONS) {
		freelog(LOG_NORMAL, "Received bad nation id %d in handle_ruleset_nation()",
				p->id);
		return;
	}
	pl = &nations[p->id];

	sz_strlcpy(pl->name, p->name);
	sz_strlcpy(pl->name_plural, p->name_plural);
	sz_strlcpy(pl->flag_graphic_str, p->graphic_str);
	sz_strlcpy(pl->flag_graphic_alt, p->graphic_alt);
	pl->leader_count = p->leader_count;
	for( i=0; i<pl->leader_count; i++) {
		pl->leader_name[i] = mystrdup(p->leader_name[i]);
		pl->leader_is_male[i] = p->leader_sex[i];
	}
	pl->city_style = p->city_style;
	freelog(LOG_DEBUG, "added '%s' under index %d",p->name,p->id);
}

struct nation_type *find_nation_by_id(Nation_Type_id id)
{
	if(id < 0 || id >= nations_count || id >= MAX_NUM_NATIONS)
		return NULL;
	return &nations[id];
}

int nation_exists(Nation_Type_id nation)
{
	return find_nation_by_id(nation)!=NULL;
}
