
#include <assert.h>

#include "priv_player.h"
#include "game.h"
#include "support.h"
#include "aiclient.h"
#include "callback.h"
#include "log.h"
#include "net.h"
#include "government.h"

choose_science_cbt cb_choose_science;
choose_government_cbt cb_choose_government;
struct player players[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
int players_count=MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS;

void set_choose_science_cb(choose_science_cbt callback)
{
	cb_choose_science=callback;
}

void set_choose_government_cb(choose_government_cbt callback)
{
	cb_choose_government=callback;
}

void update_research(struct player *plr)
{
  int i, known;
  
  for (i=0;i<get_techs_count();i++) {
	  if(!tech_exists(i)) {
		  plr->research.inventions[i]=TECH_UNKNOWN;
		  continue;
	  }

	  known=get_invention(plr->id, i);
	  if (known == TECH_REACHABLE || known == TECH_MARKED)
		  plr->research.inventions[i]=TECH_UNKNOWN;
	  if (get_invention(plr->id, i) == TECH_UNKNOWN
		  && get_invention(plr->id, find_tech_by_id(i)->req[0])==TECH_KNOWN   
		  && get_invention(plr->id, find_tech_by_id(i)->req[1])==TECH_KNOWN)
	  {
		  plr->research.inventions[i]=TECH_REACHABLE;
	  }
  }
}

int get_invention(int player_id, int tech)
{
	if(!tech_exists(tech))
		return TECH_UNKNOWN;
	return find_player_by_id(player_id)->research.inventions[tech];
}

void update_player_info(struct packet_player_info *p)
{
	int i;
	int poptechup=0;
	struct player *pplayer=&players[p->playerno];

	pplayer->id=p->playerno;
	sz_strlcpy(pplayer->name, p->name);
	pplayer->nation=p->nation;
	pplayer->is_male=p->is_male;

	pplayer->economic.gold=p->gold;
	pplayer->economic.tax=p->tax;
	pplayer->economic.science=p->science;
	pplayer->economic.luxury=p->luxury;
	pplayer->government=p->government;
	pplayer->embassy=p->embassy;
	pplayer->city_style=p->city_style;

	for (i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
		pplayer->diplstates[i].type =
			p->diplstates[i].type;
		pplayer->diplstates[i].turns_left =
			p->diplstates[i].turns_left;
		pplayer->diplstates[i].has_reason_to_cancel =
			p->diplstates[i].has_reason_to_cancel;
	}
	pplayer->reputation = p->reputation;

	for(i=0; i<get_techs_count(); i++)
		pplayer->research.inventions[i]=p->inventions[i]-'0';
	update_research(pplayer);

	if(pplayer->research.researching!=p->researching)
	{
		poptechup = 1;
		pplayer->research.tech_last_researched=pplayer->research.researching;
	}

	pplayer->research.researched=p->researched;
	pplayer->research.researched_techs=p->researchpoints;
	pplayer->research.researching=p->researching;
	pplayer->research.tech_goal=p->tech_goal;
	pplayer->research.future_tech=p->future_tech;

#if 0
	if(!pplayer->conn){
		/* It is only the client that does this */

		pplayer->conn = fc_malloc(sizeof(struct connection));
		pplayer->conn->sock = 0;
		pplayer->conn->used = 0;
		pplayer->conn->player = NULL;
	}

	sz_strlcpy(pplayer->conn->capability, p->capability);
#endif
  
	poptechup=(get_client_state()==CLIENT_GAME_RUNNING_STATE && 
			   p->playerno==game.player_id &&
			   poptechup);

	pplayer->turn_done=p->turn_done;
	pplayer->nturns_idle=p->nturns_idle;
	pplayer->is_alive=p->is_alive;
  
	pplayer->is_connected=p->is_connected;
	sz_strlcpy(pplayer->addr, p->addr);

	pplayer->revolution=p->revolution;

	/* calculate some data */

	pplayer->research.researchpoints=((game.year>0) ? 2:1)*
		pplayer->research.researched_techs*game.techlevel;

	if(p->playerno==game.player_id && 
	   (pplayer->revolution < 1 || pplayer->revolution > 5) && 
	   pplayer->government==game.government_when_anarchy &&
	   (get_client_state()==CLIENT_GAME_RUNNING_STATE))
	{
		struct packet_player_request packet;

		packet.government=pplayer->government;

		CALL(cb_choose_government,&packet.government);
		send_packet_player_request(&aconnection, &packet, 
								   PACKET_PLAYER_GOVERNMENT);
	}

	if(poptechup)
		call_choose_science();
}

void call_choose_science()
{
	struct player *pplayer=find_myself();
	struct packet_player_request packet_tech;
	struct packet_player_request packet_tech_goal;

	packet_tech.tech=pplayer->research.researching;
	packet_tech_goal.tech=pplayer->research.tech_goal;
	
	CALL(cb_choose_science, &packet_tech.tech,
		 &packet_tech_goal.tech);

	send_packet_player_request(&aconnection, &packet_tech_goal,
							   PACKET_PLAYER_TECH_GOAL);
	send_packet_player_request(&aconnection, &packet_tech,
							   PACKET_PLAYER_RESEARCH);
}

struct player *find_player_by_id(int player_id)
{
	if(player_id<0 || player_id >= players_count)
		return NULL;
	return &players[player_id];
}

struct player *find_myself()
{
	return find_player_by_id(game.player_id);
}

int player_exists(int player_id)
{
	return find_player_by_id(player_id)!=NULL;
}

int player_knows_tech_with_flag(int player_id, int flag)
{
	int i;
	int count=0;

	for( i=A_FIRST; i<get_techs_count(); i++ ) {
		if((get_invention(player_id, i) == TECH_KNOWN) && 
		   tech_has_flag(i,flag))
			count++;
	}
	return count;
}

int can_player_build_unit_type_direct(int player_id, Unit_Type_id id)
{
	if (!unit_type_exists(id))
		return 0;
	if (unit_type_has_flag(id, F_NUCLEAR) && 
		!game.global_wonders[B_MANHATTEN])
		return 0;
	if (get_invention(player_id,
					  find_unit_type_by_id(id)->tech_requirement)!=TECH_KNOWN)
		return 0;
	return 1;
}

int player_owns_active_wonder(int player_id,Impr_Type_id id)
{
	struct impr_type *ptype=find_improvement_type_by_id(id);
	struct city *pcity=find_city_by_id(game.global_wonders[id]);
	
	return (ptype &&
			ptype->is_wonder  && 
			(!wonder_obsolete(id)) && 
			pcity &&
			pcity->owner==player_id);
}

int player_owns_active_govchange_wonder(int player_id)
{
	return ( player_owns_active_wonder(player_id, B_LIBERTY) ||
			 ( (find_improvement_type_by_id(B_PYRAMIDS)->variant==1) &&
			   player_owns_active_wonder(player_id, B_PYRAMIDS) ) ||
			 ( (find_improvement_type_by_id(B_UNITED)->variant==1) &&
			   player_owns_active_wonder(player_id, B_UNITED) ) );
}

int player_can_change_to_government(int player_id, int government)
{
	int req;

	assert(government_exists(government));

	req = find_government_by_id(government)->required_tech;
	if (!tech_exists(req))
		return 0;

	return (req == A_NONE || 
			(get_invention(player_id, req) == TECH_KNOWN) || 
			player_owns_active_govchange_wonder(player_id));
}

