
#include <assert.h>

#include "mem.h"
#include "priv_tech.h"
#include "log.h"
#include "support.h"

struct advance *techs=NULL;
int techs_count=0;

void free_techs()
{
	if(!techs || !techs_count)
		return;

	free(techs);
	techs=NULL;
	techs_count=0;
}

void alloc_techs(int size)
{
	techs = fc_calloc(size, sizeof(struct advance));
	techs_count=size;
}

int get_techs_count()
{
	return techs_count;
}

void add_tech(struct packet_ruleset_tech *p)
{
	struct advance *a;

	if(p->id < 0 || p->id >= techs_count || p->id >= A_LAST) {
		freelog(LOG_NORMAL, 
				"Received bad advance id %d in handle_ruleset_tech()",
				p->id);
		return;
	}
	a = &techs[p->id];

	a->id=p->id;
	sz_strlcpy(a->name, p->name);
	a->req[0] = p->req[0];
	a->req[1] = p->req[1];
	a->flags = p->flags;
  
	free(a->helptext);
	a->helptext = p->helptext;	/* pointer assignment */
	freelog(LOG_DEBUG,"added '%s' under index %d",p->name,p->id);
}

struct advance *find_tech_by_id(int id)
{
	if(id < 0 || id >= techs_count || id >= A_LAST)
		return NULL;
	else if (techs[id].req[0]==A_LAST || 
			 techs[id].req[1]==A_LAST)
		return NULL;

	return &techs[id];
}

struct advance *find_tech_by_name(const char *name)
{
	int i;

	for(i=0;i<techs_count;i++)
	{
		if(strcmp(techs[i].name,name)==0)
			return &techs[i];
	}
	return NULL;
}

int tech_exists(Tech_Type_id id)
{
	return find_tech_by_id(id)!=NULL;
}

int tech_has_flag(int tech_id, int flag)
{
	assert(flag>=0 && flag<TF_LAST);
	return BOOL_VAL(find_tech_by_id(tech_id)->flags & (1<<flag));
}
