/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifndef FC__CITY_H
#define FC__CITY_H

struct city;
struct government;
struct unit;

#include "tech.h"
#include "unit.h"
#include "nation.h"
#include "map.h"


/* Improvement types.
 * (This will later not be an enum.)
 */
enum improvement_type_id {
	B_AIRPORT=0, B_AQUEDUCT, B_BANK, B_BARRACKS, B_BARRACKS2, B_BARRACKS3, 
	B_CATHEDRAL, B_CITY, B_COASTAL, B_COLOSSEUM, B_COURTHOUSE,  B_FACTORY, 
	B_GRANARY, B_HARBOUR, B_HYDRO, B_LIBRARY, B_MARKETPLACE, B_MASS, B_MFG, 
	B_NUCLEAR, B_OFFSHORE, B_PALACE, B_POLICE, B_PORT, B_POWER,
	B_RECYCLING, B_RESEARCH, B_SAM, B_SDI, B_SEWER, B_SOLAR, B_SCOMP, 
	B_SMODULE, B_SSTRUCTURAL, B_STOCK, B_SUPERHIGHWAYS, B_SUPERMARKET, 
	B_TEMPLE, B_UNIVERSITY,  
  
	B_APOLLO, B_ASMITHS, B_COLLOSSUS, B_COPERNICUS, B_CURE, B_DARWIN, B_EIFFEL,
	B_GREAT, B_WALL, B_HANGING, B_HOOVER, B_ISAAC, B_BACH, B_RICHARDS, 
	B_LEONARDO, B_LIGHTHOUSE, B_MAGELLAN, B_MANHATTEN, B_MARCO, 
	B_MICHELANGELO, 

	B_ORACLE, B_PYRAMIDS, B_SETI, B_SHAKESPEARE, B_LIBERTY, B_SUNTZU, 
	B_UNITED, B_WOMENS,
	B_CAPITAL, B_LAST
};

typedef int Impr_Type_id;

/* Range of effects.
 * Used in equiv_range and effect.range fields.
 * (These must correspond to effect_range_names[] in city.c.)
 */
enum effect_range_id {
	EFR_NONE,
	EFR_BUILDING,
	EFR_CITY,
	EFR_ISLAND,
	EFR_PLAYER,
	EFR_WORLD,
	EFR_LAST	/* keep this last */
};

typedef enum effect_range_id Eff_Range_id;

/* Type of effects.
 * Used in effect.type field.
 * (These must correspond to effect_type_names[] in city.c.)
 */
enum effect_type_id {
	EFT_ADV_PARASITE,
	EFT_AIRLIFT,
	EFT_ANY_GOVERNMENT,
	EFT_BARB_ATTACK,
	EFT_BARB_DEFEND,
	EFT_CAPITAL_CITY,
	EFT_CAPITAL_EXISTS,
	EFT_ENABLE_NUKE,
	EFT_ENABLE_SPACE,
	EFT_ENEMY_PEACEFUL,
	EFT_FOOD_ADD_TILE,
	EFT_FOOD_BONUS,
	EFT_FOOD_INC_TILE,
	EFT_FOOD_PER_TILE,
	EFT_GIVE_IMM_ADV,
	EFT_GROWTH_FOOD,
	EFT_HAVE_EMBASSIES,
	EFT_IMPROVE_REP,
	EFT_LUXURY_BONUS,
	EFT_LUXURY_PCT,
	EFT_MAKE_CONTENT,
	EFT_MAKE_CONTENT_MIL,
	EFT_MAKE_CONTENT_PCT,
	EFT_MAKE_HAPPY,
	EFT_MAY_DECLARE_WAR,
	EFT_NO_ANARCHY,
	EFT_NO_SINK_DEEP,
	EFT_NUKE_PROOF,
	EFT_POLLU_ADJ,
	EFT_POLLU_ADJ_POP,
	EFT_POLLU_ADJ_PROD,
	EFT_POLLU_DECREASE,
	EFT_PROD_ADD_TILE,
	EFT_PROD_BONUS,
	EFT_PROD_INC_TILE,
	EFT_PROD_PER_TILE,
	EFT_PROD_TO_GOLD,
	EFT_REDUCE_CORRUPT,
	EFT_REDUCE_WASTE,
	EFT_REVEAL_CITIES,
	EFT_REVEAL_MAP,
	EFT_REVOLT_DIST,
	EFT_SCIENCE_BONUS,
	EFT_SCIENCE_PCT,
	EFT_SIZE_UNLIMIT,
	EFT_SLOW_GLOBAL_WARM,
	EFT_SPACE_PART,
	EFT_SPY_RESISTANT,
	EFT_TAX_BONUS,
	EFT_TAX_PCT,
	EFT_TRADE_ADD_TILE,
	EFT_TRADE_BONUS,
	EFT_TRADE_INC_TILE,
	EFT_TRADE_PER_TILE,
	EFT_TRADE_ROUTE_PCT,
	EFT_UNIT_DEFEND,
	EFT_UNIT_MOVE,
	EFT_UNIT_NO_LOSE_POP,
	EFT_UNIT_RECOVER,
	EFT_UNIT_REPAIR,
	EFT_UNIT_VET_COMBAT,
	EFT_UNIT_VETERAN,
	EFT_UPGRADE_UNITS,
	EFT_UPGRADE_ONE_STEP,
	EFT_UPGRADE_ONE_LEAP,
	EFT_UPGRADE_ALL_STEP,
	EFT_UPGRADE_ALL_LEAP,
	EFT_UPKEEP_FREE,
	EFT_LAST	/* keep this last */
};

typedef enum effect_type_id Eff_Type_id;

/* An effect conferred by an improvement.
 */
struct impr_effect {
	Eff_Type_id type;
	Eff_Range_id range;
	int amount;
	Impr_Type_id cond_bldg;		/* B_LAST = unconditional */
	int cond_gov;				/* game.government_count = unconditional */
	Tech_Type_id cond_adv;		/* A_NONE = unconditional; A_LAST = never */
	Eff_Type_id cond_eff;			/* EFT_LAST = unconditional */
	Unit_Class_id aff_unit;		/* UCL_LAST = all */
	enum tile_terrain_type aff_terr;	/* T_UNKNOWN = all; T_LAST = none */
	enum tile_special_type aff_spec;	/* S_* bit mask of specials affected */
};

/* Type of improvement.
 * (Read from buildings.ruleset file.)
 */
struct impr_type {
	char name[MAX_LEN_NAME];
	char name_orig[MAX_LEN_NAME];		/* untranslated */
	Tech_Type_id tech_req;		/* A_LAST = never; A_NONE = always */
	Impr_Type_id bldg_req;		/* B_LAST = none required */
	enum tile_terrain_type *terr_gate;	/* list; T_LAST terminated */
	enum tile_special_type *spec_gate;	/* list; S_NO_SPECIAL terminated */
	Eff_Range_id equiv_range;
	Impr_Type_id *equiv_dupl;		/* list; B_LAST terminated */
	Impr_Type_id *equiv_repl;		/* list; B_LAST terminated */
	Tech_Type_id obsolete_by;		/* A_NONE = never obsolete */
	int is_wonder;
	int build_cost;
	int upkeep;
	int sabotage;
	struct impr_effect *effect;		/* list; .type==EFT_LAST terminated */
	int variant;			/* FIXME: remove when gen-impr obsoletes */
	char *helptext;

	/* calculated values below */
	int obsolete; /* normal obsolete and wonder obs. */
	int is_wonder_useful;
	int exists;
	int final_upkeep;
};

enum specialist_type {
	SP_ELVIS, SP_SCIENTIST, SP_TAXMAN
};

enum city_tile_type {
	C_TILE_EMPTY, C_TILE_WORKER, C_TILE_UNAVAILABLE
};

enum city_options {
	/* The first 4 are whether to auto-attack versus each unit move_type
	 * from with auto-attack units within this city.  Note that these
	 * should stay the first four, and must stay in the same order as
	 * enum unit_move_type.  
   *
   * The next is whether building a settler at size 1 disbands a city.
   *
   * The following 2 are what to do of new citizens when the city grows:
   * make them workers, scientists, or taxmen. Should have only one set,
   * or if neither is set, that means make workers.
   *
   * Any more than 8 options requires a protocol extension, since
   * we only send 8 bits.
   */
	CITYO_ATT_LAND=0, CITYO_ATT_SEA, CITYO_ATT_HELI, CITYO_ATT_AIR,
	CITYO_DISBAND, CITYO_NEW_EINSTEIN, CITYO_NEW_TAXMAN
};

/* first four bits are for auto-attack: */
#define CITYOPT_AUTOATTACK_BITS 0xF

/* for new city: default auto-attack options all on, others off: */
#define CITYOPT_DEFAULT (CITYOPT_AUTOATTACK_BITS)

struct city_tile
{
	int shields;
	int trade;
	int food;
	enum city_tile_type type;
};

#define CITY_MAP_SIZE 5

struct city {
	int id;
	int owner;
	int x, y;
	char name[MAX_LEN_NAME];

	int size;
	int ppl_happy, ppl_content, ppl_unhappy;
	int ppl_elvis, ppl_scientist, ppl_taxman;
	int food_prod, food_surplus;
	int shield_prod, shield_surplus;
	int trade_prod, corruption;
	int trade[4],trade_value[4];
	int luxury_total, tax_total, science_total;

	/* the physics */
	int food_stock;
	int shield_stock;
	int pollution;

	int is_building_unit;
	int currently_building;

	char improvements[B_LAST+1];
	struct city_tile city_map[CITY_MAP_SIZE][CITY_MAP_SIZE];

	int did_buy, did_sell;
	int was_happy;
	int airlift;
	int diplomat_investigate;
	int city_options;

	/* calculated values below */
	int is_building_wonder;
	int gold_surplus;
	int buy_cost;
	int is_happy, is_unhappy, is_celebrating, is_rapture_growing;
	int got_citywalls;
	int turns_to_complete;
};

struct city *find_city_by_id(int id);
struct city *find_city_by_name(char *name);
struct city *find_city_by_position(int x, int y);
int find_cities_by_player(int player_id, int store_size, struct city **store);
int city_exists(int city_id);

struct city_style {
  char name[MAX_LEN_NAME];
  char name_orig[MAX_LEN_NAME];	      /* untranslated */
  int techreq;                  /* tech required to use a style      */
  int replaced_by;              /* index to replacing style          */
};

/* improvement functions */

struct impr_type *find_improvement_type_by_name(char *s);
struct impr_type *find_improvement_type_by_id(Impr_Type_id id);
int improvement_type_exists(Impr_Type_id id);

/* player related improvement and unit functions */

int can_player_build_unit_type_direct(int player_id, Unit_Type_id id);
int wonder_obsolete(Impr_Type_id id); /* RFFIXME struct member */

/* general city functions */

void set_construction_to_unit(int city_id, Unit_Type_id id);
void set_construction_to_improvement(int city_id,Impr_Type_id id);
void rename_city(int city_id, char *new_name);
void buy_city_construction(int city_id);
void sell_city_improvement(int city_id, int improvement_id);
void set_city_map(int city_id, struct city_tile *city_tiles);

/* city related improvement and unit functions */

int city_got_improvement(int city_id, Impr_Type_id id);
int can_city_build_unit_type(int city_id, Unit_Type_id id);
int city_affected_by_wonder(struct city *pcity, Impr_Type_id id);

/* Callbacks */

typedef void (*city_changed_notify_cbt)(int city_id);
void set_city_changed_notify_cb(city_changed_notify_cbt callback);

typedef void (*new_city_notify_cbt)(int city_id);
void set_new_city_notify_cb(new_city_notify_cbt callback);

typedef void (*city_destroyed_notify_cbt)(int city_id);
void set_city_destroyed_notify_cb(city_destroyed_notify_cbt callback);

typedef void (*city_construction_finished_notify_cbt)(int city_id);
void set_city_construction_finished_notify_cb(city_construction_finished_notify_cbt callback);

/* trade functions */

int can_establish_trade_route(int city1_id, int city2_id);

/* Unused functions */

#if 0
int could_build_improvement(struct city *pcity, Impr_Type_id id);
int can_build_improvement(struct city *pcity, Impr_Type_id id);
int can_eventually_build_improvement(struct city *pcity, Impr_Type_id id);

int can_build_unit_direct(struct city *pcity, Unit_Type_id id);
int can_eventually_build_unit(struct city *pcity, Unit_Type_id id);
int city_got_building(struct city *pcity,  Impr_Type_id id); 

int city_got_effect(struct city *pcity, Impr_Type_id id);
int wonder_replacement(struct city *pcity, Impr_Type_id id);

int city_turns_to_build(struct city *pcity, int id, int id_is_unit);
int could_player_eventually_build_improvement(Impr_Type_id id);
int could_player_build_improvement(Impr_Type_id id);
int can_player_build_improvement(Impr_Type_id id);
int can_player_build_unit(Unit_Type_id id);
int can_player_eventually_build_unit(Unit_Type_id id);

int trade_between_cities(int city1_id, int city2_id);
int city_num_trade_routes(int city_id);

int citygov_free_shield(struct city *pcity, struct government *gov);
int citygov_free_happy(struct city *pcity, struct government *gov);
int citygov_free_food(struct city *pcity, struct government *gov);
int citygov_free_gold(struct city *pcity, struct government *gov);
#endif

#endif  /* FC__CITY_H */
