/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifndef FC__GAME_H
#define FC__GAME_H

#include <time.h>	/* time_t */
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include "shared.h"
#include "player.h"

#define MAX_LEN_DEMOGRAPHY  16

enum server_states { 
  PRE_GAME_STATE, 
  SELECT_RACES_STATE, 
  RUN_GAME_STATE,
  GAME_OVER_STATE
};

enum client_states { 
  CLIENT_BOOT_STATE,
  CLIENT_PRE_GAME_STATE,
  CLIENT_SELECT_RACE_STATE,
  CLIENT_WAITING_FOR_GAME_START_STATE,
  CLIENT_GAME_RUNNING_STATE
};

struct unit;
struct city;

struct civ_game {
  int start_gold;
  int civstyle;
  int tech;
  int techlevel;
  int skill_level;
  int timeout;
  int end_year;
  int year;
  int min_players, max_players, nplayers;
  int player_id;
  int globalwarming;
  int heating;
  int cityfactor;
  int unhappysize;
  int diplcost,freecost,conquercost;
  int global_advances[A_LAST];
  int global_wonders[B_LAST];
  int foodbox;
  int techpenalty;
  int spacerace;
  int seconds_to_turndone;

  struct {
    int get_bonus_tech;
    int cathedral_plus;
    int cathedral_minus;
    int colosseum_plus;
    int temple_plus;
    int partisan_req[MAX_NUM_TECH_LIST]; 
  } rtech;

	int aqueduct_size, sewer_size, add_to_size_limit;
	int government_when_anarchy, default_government;
	int playable_nation_count;

	/* calculated values */

#if 0
	int diplchance;
	int civilwarsize;
	
  int warminglevel;
  int aqueductloss;
  int killcitizen;
  int razechance;
  int scorelog;
  int randseed;
  int aqueduct_size;
  int sewer_size;
  int add_to_size_limit;
  int turnblock;
  int fixedlength;
  int auto_ai_toggle;
  int fogofwar;
  int fogofwar_old;	/* as the fog_of_war bit get changed by setting
			   the server we need to remember the old setting */

  int num_unit_types;
  int num_tech_types;  /* including A_NONE */

  int government_count;
  int default_government;
  int government_when_anarchy;
  int ai_goal_government;	/* kludge */

  int nation_count;
  int playable_nation_count;
  int styles_count;
#endif
};

extern struct civ_game game;

void game_init(void);
void turn_done(void);
void make_revolution(void);

/* Callbacks */

typedef void (*select_nation_cbt)(int *nation_id_store, int *is_male_store,
								  int *city_style_store, 
								  char *ruler_name_store);
void set_select_nation_cb(select_nation_cbt callback);

typedef void (*game_started_notify_cbt)();
void set_game_started_notify_cb(game_started_notify_cbt callback);

typedef void (*new_year_notify_cbt)();
void set_new_year_notify_cb(new_year_notify_cbt callback);

typedef void (*name_new_city_cbt)(int unit_id, char *suggested_name, 
								  char *name_store);
void set_name_new_city_cb(name_new_city_cbt callback);

#endif  /* FC__GAME_H */
