/********************************************************************** 
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifndef FC__MAP_H
#define FC__MAP_H

#include "nation.h"
#include "terrain.h"

struct unit;

#define NUM_DIRECTION_NSEW 16

struct tile {
	enum tile_terrain_type terrain;
	enum tile_special_type special;
	int known;

	/* calculated values */
	signed short continent;
};

/****************************************************************
miscellaneous terrain information
*****************************************************************/

struct terrain_misc
{
  /* options */
  enum terrain_river_type river_style;
  int may_road;       /* boolean: may build roads/railroads */
  int may_irrigate;   /* boolean: may build irrigation/farmland */
  int may_mine;       /* boolean: may build mines */
  int may_transform;  /* boolean: may transform terrain */
  /* parameters */
  int ocean_reclaim_requirement;       /* # adjacent land tiles for reclaim */
  enum special_river_move river_move_mode;
  int river_defense_bonus;             /* % added to defense if Civ2 river */
  int river_trade_incr;                /* added to trade if Civ2 river */
  char *river_help_text;	       /* help for Civ2-style rivers */
  int fortress_defense_bonus;          /* % added to defense if fortress */
  int road_superhighway_trade_bonus;   /* % added to trade if road/s-highway */
  int rail_food_bonus;                 /* % added to food if railroad */
  int rail_shield_bonus;               /* % added to shield if railroad */
  int rail_trade_bonus;                /* % added to trade if railroad */
  int farmland_supermarket_food_bonus; /* % added to food if farm/s-market */
  int pollution_food_penalty;          /* % subtr. from food if polluted */
  int pollution_shield_penalty;        /* % subtr. from shield if polluted */
  int pollution_trade_penalty;         /* % subtr. from trade if polluted */
};

/****************************************************************
tile_type for each terrain type
expand with government bonuses??
*****************************************************************/

struct tile_type {
  char terrain_name[MAX_LEN_NAME];     /* "" if unused */
  char terrain_name_orig[MAX_LEN_NAME];	/* untranslated copy */

  int movement_cost;
  int defense_bonus;

  int food;
  int shield;
  int trade;

  char special_1_name[MAX_LEN_NAME];   /* "" if none */
  char special_1_name_orig[MAX_LEN_NAME]; /* untranslated copy */
  int food_special_1;
  int shield_special_1;
  int trade_special_1;

  char special_2_name[MAX_LEN_NAME];   /* "" if none */
  char special_2_name_orig[MAX_LEN_NAME]; /* untranslated copy */
  int food_special_2;
  int shield_special_2;
  int trade_special_2;

  int road_trade_incr;
  int road_time;

  enum tile_terrain_type irrigation_result;
  int irrigation_food_incr;
  int irrigation_time;

  enum tile_terrain_type mining_result;
  int mining_shield_incr;
  int mining_time;

  enum tile_terrain_type transform_result;
  int transform_time;

  char *helptext;
};

struct tile *find_tile_by_position(int x, int y);

struct tile_type *find_tile_type_by_id(enum tile_terrain_type type);
int tile_type_exists(enum tile_terrain_type type);

int same_pos(int x1, int y1, int x2, int y2);
int map_distance(int x0, int y0, int x1, int y1);
int is_tiles_adjacent(int x0, int y0, int x1, int y1);
int is_terrain_near_tile(int x, int y, enum tile_terrain_type t);
int is_irrigation_water_adjacent(int x, int y);
int get_tile_infrastructure_set(enum tile_special_type tile_special);
int get_unit_tile_pillage_set(int x, int y);
int get_preferred_pillage(int pset);
int map_move_cost(int unit_id, int x, int y);

#define map_adjust_x(X) \
 (((X)<0) ?  (X)+map.xsize : (((X)>=map.xsize) ? (X)-map.xsize : (X)))
#define map_adjust_y(Y) \
  (((Y)<0) ? 0 : (((Y)>=map.ysize) ? map.ysize-1 : (Y)))

/* Unused functions */

#if 0
int real_map_distance(int x0, int y0, int x1, int y1);
int sq_map_distance(int x0, int y0, int x1, int y1);

int map_same_continent(int x1, int y1, int x2, int y2);

void initialize_move_costs(void);
void reset_move_costs(int x, int y);
char *map_get_tile_info_text(int x, int y);

#define map_inx(x,y) \
  ((x)+(y)*map.xsize)

void map_calc_adjacent_xy(int x, int y, int *xx, int *yy);
void map_calc_adjacent_xy_void(int x, int y, int *xx, int *yy);

void map_set_terrain(int x, int y, enum tile_terrain_type ter);
void map_set_special(int x, int y, enum tile_special_type spe);
void map_clear_special(int x, int y, enum tile_special_type spe);
int is_real_tile(int x, int y);
int normalize_map_pos(int *x, int *y);

enum tile_terrain_type get_terrain_by_name(char * name);
char *get_terrain_name(enum tile_terrain_type type);
enum tile_special_type get_special_by_name(char * name);
char *get_special_name(enum tile_special_type type);

int isnt_terrain_near_tile(int x, int y, enum tile_terrain_type t);
int count_terrain_near_tile(int x, int y, enum tile_terrain_type t);
int is_special_near_tile(int x, int y, enum tile_special_type spe);
int count_special_near_tile(int x, int y, enum tile_special_type spe);
int is_coastline(int x,int y);
int terrain_is_clean(int x, int y);
int is_at_coast(int x, int y);
int is_water_adjacent(int x, int y);
int is_hut_close(int x, int y);
int is_starter_close(int x, int y, int nr, int dist); 
int is_good_tile(int x, int y);
int is_special_close(int x, int y);
int is_special_type_close(int x, int y, enum tile_special_type spe);
int is_sea_usable(int x, int y);
int get_tile_food_base(struct tile * ptile);
int get_tile_shield_base(struct tile * ptile);
int get_tile_trade_base(struct tile * ptile);
char *map_get_infrastructure_text(int spe);
int map_get_infrastructure_prerequisite(int spe);

void map_irrigate_tile(int x, int y);
void map_mine_tile(int x, int y);
void map_transform_tile(int x, int y);

int map_build_road_time(int x, int y);
int map_build_irrigation_time(int x, int y);
int map_build_mine_time(int x, int y);
int map_transform_time(int x, int y);

/* This iterates outwards from the starting point (Duh?).
   Every tile within max_dist will show up exactly once. (even takes
   into account wrap). All positions given correspond to real tiles.
   The values given are adjusted.
   You should make sure that the arguments passed to the macro are adjusted,
   or you could have some very nasty intermediate errors.
   The results are returned through ARG_x_itr and ARG_y_itr,
   which must already be defined.

   Internally it works by for a given distance
   1) assume y positive and iterate over x
   2) assume y negative and iterate over x
   3) assume x positive and iterate over y
   4) assume x negative and iterate over y
   Where in this we are is decided by the variables xcycle and positive.
   each of there distances give a box of tiles; to ensure each tile is only
   returned once we only return the corner when iterating over x.
   As a special case positive is initialized as 0 (ie we start in step 2) ),
   as the center tile would else be returned by both step 1) and 2).
*/
#define iterate_outward(ARG_start_x, ARG_start_y, ARG_max_dist, ARG_x_itr, ARG_y_itr) \
{                                                                             \
  int MACRO_max_dx = map.xsize/2;                                             \
  int MACRO_min_dx = -(MACRO_max_dx - (map.xsize%2 ? 0 : 1));                 \
  int MACRO_xcycle = 1;                                                       \
  int MACRO_positive = 0;                                                     \
  int MACRO_dxy = 0, MACRO_do_xy;                                             \
  while(MACRO_dxy <= ARG_max_dist) {                                          \
    for (MACRO_do_xy = -MACRO_dxy; MACRO_do_xy <= MACRO_dxy; MACRO_do_xy++) { \
      if (MACRO_xcycle) {                                                     \
	ARG_x_itr = ARG_start_x + MACRO_do_xy;                                \
	if (MACRO_positive)                                                   \
	  ARG_y_itr = ARG_start_y + MACRO_dxy;                                \
	else                                                                  \
	  ARG_y_itr = ARG_start_y - MACRO_dxy;                                \
      } else { /* ! MACRO_xcycle */                                           \
        if (MACRO_dxy == MACRO_do_xy || MACRO_dxy == -MACRO_do_xy)            \
          continue;                                                           \
	ARG_y_itr = ARG_start_y + MACRO_do_xy;                                \
	if (MACRO_positive)                                                   \
	  ARG_x_itr = ARG_start_x + MACRO_dxy;                                \
	else                                                                  \
	  ARG_x_itr = ARG_start_x - MACRO_dxy;                                \
      }                                                                       \
      if (ARG_y_itr<0 || ARG_y_itr >= map.ysize)                              \
	continue;                                                             \
      {                                                                       \
	int MACRO_dx = ARG_start_x - ARG_x_itr;                               \
	if (MACRO_dx > MACRO_max_dx || MACRO_dx < MACRO_min_dx)               \
	  continue;                                                           \
      }                                                                       \
      if (ARG_x_itr > map.xsize)                                              \
	ARG_x_itr -= map.xsize;                                               \
      else if (ARG_x_itr < 0)                                                 \
	ARG_x_itr += map.xsize;

#define iterate_outward_end                                                   \
    }                                                                         \
    if (!MACRO_positive) {                                                    \
      if (!MACRO_xcycle)                                                      \
	MACRO_dxy++;                                                          \
      MACRO_xcycle = !MACRO_xcycle;                                           \
    }                                                                         \
    MACRO_positive = !MACRO_positive;                                         \
  }                                                                           \
}

/* Iterate through all tiles in a square with given center and radius.
   Positions returned will have adjusted x, and positions with illegal y will be
   automatically discarded.
 */
#define square_iterate(SI_center_x, SI_center_y, radius, SI_x_itr, SI_y_itr)  \
{                                                                             \
  int SI_x_itr1;                                                              \
  for (SI_y_itr = SI_center_y - radius;                                       \
       SI_y_itr <= SI_center_y + radius; SI_y_itr++) {                        \
    if (SI_y_itr < 0 || SI_y_itr >= map.ysize)                                \
      continue;                                                               \
    for (SI_x_itr1 = SI_center_x - radius;                                    \
	 SI_x_itr1 <= SI_center_x + radius; SI_x_itr1++) {                    \
      SI_x_itr = map_adjust_x(SI_x_itr1);

#define square_iterate_end                                                    \
    }                                                                         \
  }                                                                           \
}
#endif

#endif  /* FC__MAP_H */
